import os

test_data_dir = os.path.join(os.path.dirname(__file__), 'data')
# pdf_name = 'plan_l.pdf'
# pdf_name = 'plan_p.pdf'
# pdf_name = 'article.pdf'
pdf_name = 'document.pdf'
# pdf_name = 'con docs2.pdf'
img_name = 'floor plan.png'
pdf_path = os.path.join(test_data_dir, pdf_name)
img_path = os.path.join(test_data_dir, img_name)

__all__ = ['pdf_path', 'img_path', 'test_data_dir']



from pdfconduit.modify.canvas.constructor import CanvasConstructor
from pdfconduit.modify.canvas.objects import CanvasImg, CanvasStr, CanvasObjects

__all__ = ['CanvasConstructor', 'CanvasImg', 'CanvasStr', 'CanvasObjects']



from pdfconduit.transform.merge import Merge
from pdfconduit.transform.rotate import rotate, Rotate
from pdfconduit.transform.slice import slicer
from pdfconduit.transform.upscale import upscale, Upscale

__all__ = ['Merge', 'rotate', 'Rotate', 'slicer', 'upscale', 'Upscale']



# Rotate a pdf file
import os
from tempfile import NamedTemporaryFile

from PyPDF3 import PdfFileReader, PdfFileWriter
from pdfrw import PdfReader, PdfWriter

from pdfconduit.utils.path import add_suffix


class Rotate:
    def __init__(self, file_name, rotation, suffix='rotated', tempdir=None, method='pdfrw'):
        self.file_name = file_name
        self.rotation = rotation
        self.suffix = suffix
        self.tempdir = tempdir

        if tempdir:
            with NamedTemporaryFile(suffix='.pdf', dir=tempdir, delete=False) as temp:
                self.outfn = temp.name
        elif suffix:
            self.outfn = os.path.join(os.path.dirname(file_name), add_suffix(file_name, suffix))
        else:
            self.outfn = NamedTemporaryFile(suffix='.pdf').name

        if method == 'pypdf3':
            self.pypdf3()
        else:
            self.pdfrw()

    def __str__(self):
        return self.file

    @property
    def file(self):
        return str(self.outfn)

    def pypdf3(self):
        with open(self.file_name, 'rb') as pdf_in:
            pdf_writer = PdfFileWriter()
            pdf_reader = PdfFileReader(pdf_in)
            for pagenum in range(pdf_reader.numPages):
                page = pdf_reader.getPage(pagenum)
                page.rotateClockwise(self.rotation)
                pdf_writer.addPage(page)

            with open(self.outfn, 'wb') as pdf_out:
                pdf_writer.write(pdf_out)
        return self.outfn

    def pdfrw(self):
        trailer = PdfReader(self.file_name)
        pages = trailer.pages

        ranges = [[1, len(pages)]]

        for onerange in ranges:
            onerange = (onerange + onerange[-1:])[:2]
            for pagenum in range(onerange[0] - 1, onerange[1]):
                pages[pagenum].Rotate = (int(pages[pagenum].inheritable.Rotate or 0) + self.rotation) % 360

        outdata = PdfWriter(self.outfn)
        outdata.trailer = trailer
        outdata.write()
        return self.outfn


def rotate(file_name, rotation, suffix='rotated', tempdir=None, method='pypdf3'):
    """Rotate PDF by increments of 90 degrees."""
    return str(Rotate(file_name, rotation, suffix, tempdir, method))



import os
import unittest
from tempfile import NamedTemporaryFile, TemporaryDirectory

from looptools import Timer

from pdfconduit.conduit import Encrypt, Watermark
from pdfconduit.conduit.watermark.label import Label
from pdfconduit.utils import Info
from tests import *


class TestConduitEncrypt(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        # Encryption passwords
        cls.owner_pw = 'foo'
        cls.user_pw = 'baz'
        cls.pdf_path = pdf_path

    def setUp(self):
        self.temp = NamedTemporaryFile(suffix='.pdf', delete=False)

    def tearDown(self):
        if os.path.exists(self.temp.name):
            os.remove(self.temp.name)

    @Timer.decorator
    def test_encrypt_printing(self):
        """Encrypt a PDF file and allow users to print."""
        encrypted = Encrypt(self.pdf_path, self.user_pw, self.owner_pw, output=self.temp.name, suffix='secured')

        # Encrypted pdf security info
        security = Info(encrypted.output, self.user_pw).security

        # Assert that pdf file exists
        self.assertTrue(os.path.exists(encrypted.output))

        # Assert that pdf file is now encrypted
        self.assertTrue(Info(encrypted.output, self.user_pw).encrypted)

        # Assert encryption bit size is 128
        self.assertEqual(security['/Length'], 128)

        # Assert pdf security value is -1852
        self.assertEqual(security['/P'], -1852)

    @Timer.decorator
    def test_encrypt_128bit(self):
        """Encrypt PDF file with 128bit encryption."""
        encrypted = Encrypt(self.pdf_path,
                            self.user_pw,
                            self.owner_pw,
                            output=self.temp.name,
                            bit128=True,
                            suffix='secured_128bit')

        # Encrypted pdf security info
        security = Info(encrypted.output, self.user_pw).security

        # Assert that pdf file exists
        self.assertTrue(os.path.exists(encrypted.output))

        # Assert that pdf file is now encrypted
        self.assertTrue(Info(encrypted.output, self.user_pw).encrypted)

        # Assert encryption bit size is 128
        self.assertEqual(security['/Length'], 128)

    @Timer.decorator
    def test_encrypt_40bit(self):
        """Encrypt PDF file with 40bit encryption."""
        encrypted = Encrypt(self.pdf_path,
                            self.user_pw,
                            self.owner_pw,
                            output=self.temp.name,
                            bit128=False,
                            suffix='secured_40bit')

        # Encrypted pdf security info
        security = Info(encrypted.output, self.user_pw).security

        # Assert that pdf file exists
        self.assertTrue(os.path.exists(encrypted.output))

        # Assert that pdf file is now encrypted
        self.assertTrue(Info(encrypted.output, self.user_pw).encrypted)

        # Assert pdf security value is -1852
        self.assertEqual(security['/P'], -1852)

    @Timer.decorator
    def test_encrypt_commenting(self):
        """Encrypt a PDF file but allow the user to add comments."""
        encrypted = Encrypt(self.pdf_path,
                            self.user_pw,
                            self.owner_pw,
                            output=self.temp.name,
                            allow_commenting=True,
                            suffix='secured_commenting')

        # Encrypted pdf security info
        security = Info(encrypted.output, self.user_pw).security

        # Assert that pdf file exists
        self.assertTrue(os.path.exists(encrypted.output))

        # Assert that pdf file is now encrypted
        self.assertTrue(Info(encrypted.output, self.user_pw).encrypted)

        # Assert pdf security value is -1500
        self.assertEqual(security['/P'], -1500)


class TestConduitWatermark(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.pdf_path = pdf_path

    def setUp(self):
        self.temp = TemporaryDirectory()
        self.address = '43 Indian Lane'
        self.town = 'Franklin'
        self.state = 'MA'
        self.rotate = 30
        self.owner_pw = 'foo'
        self.user_pw = 'baz'

    def tearDown(self):
        self.temp.cleanup()

    @Timer.decorator
    def test_conduit_watermark_pdfrw(self):
        """Apply a watermark to all pages of PDF using the `pdfrw` method."""
        w = Watermark(self.pdf_path, use_receipt=False, open_file=False, tempdir=self.temp.name)
        wtrmrk = w.draw(self.address,
                        str(self.town + ', ' + self.state),
                        opacity=0.08,
                        rotate=self.rotate,
                        flatten=False)
        added = w.add(self.pdf_path, wtrmrk, method='pdfrw', suffix=None)

        # Assert the watermark file exists
        self.assertTrue(os.path.exists(wtrmrk))

        # Assert watermarked PDF file exists
        self.assertTrue(os.path.exists(added))

        # Assert watermarked PDF has page resources
        self.assertTrue(Info(added).resources())
        return added

    @Timer.decorator
    def test_conduit_watermark_underneath_pdfrw(self):
        """Apply a watermark underneath original content of PDF using the `pdfrw` method."""
        w = Watermark(self.pdf_path, use_receipt=False, open_file=False, tempdir=self.temp.name)
        wtrmrk = w.draw(self.address, str(self.town + ', ' + self.state), opacity=0.08, rotate=self.rotate)
        added = w.add(self.pdf_path, wtrmrk, underneath=True, suffix=None, method='pdfrw')

        # Assert the watermark file exists
        self.assertTrue(os.path.exists(wtrmrk))

        # Assert watermarked PDF file exists
        self.assertTrue(os.path.exists(added))

        # Assert watermarked PDF has page resources
        self.assertTrue(Info(added).resources())
        return added

    @Timer.decorator
    def test_conduit_watermark_overlay_pdfrw(self):
        """Apply a watermark overlaid over original content of PDF using the `pdfrw` method."""
        w = Watermark(self.pdf_path, use_receipt=False, open_file=False, tempdir=self.temp.name)
        wtrmrk = w.draw(self.address, str(self.town + ', ' + self.state), opacity=0.08, rotate=self.rotate)
        added = w.add(self.pdf_path, wtrmrk, underneath=False, suffix=None, method='pdfrw')

        # Assert the watermark file exists
        self.assertTrue(os.path.exists(wtrmrk))

        # Assert watermarked PDF file exists
        self.assertTrue(os.path.exists(added))

        # Assert watermarked PDF has page resources
        self.assertTrue(Info(added).resources())
        return added

    @Timer.decorator
    def test_conduit_watermark_flat_pdfrw(self):
        """Apply a flattened watermark to a PDF using the `pdfrw` method."""
        w = Watermark(self.pdf_path, use_receipt=False, open_file=False, tempdir=self.temp.name)
        flat = w.draw(self.address, str(self.town + ', ' + self.state), opacity=0.08, flatten=True)
        added = w.add(self.pdf_path, flat, suffix=None, method='pdfrw')

        # Assert the watermark file exists
        self.assertTrue(os.path.exists(flat))

        # Assert watermarked PDF file exists
        self.assertTrue(os.path.exists(added))

        # Assert watermarked PDF has page resources
        self.assertTrue(Info(added).resources())
        return added

    @Timer.decorator
    def test_conduit_watermark_layered_pdfrw(self):
        """Apply a flattened watermark to a PDF using the `pdfrw` method."""
        w = Watermark(self.pdf_path, use_receipt=False, open_file=False, tempdir=self.temp.name)
        layered = w.draw(self.address, str(self.town + ', ' + self.state), opacity=0.08, flatten=False)
        added = w.add(self.pdf_path, layered, suffix=None, method='pdfrw')

        # Assert the watermark file exists
        self.assertTrue(os.path.exists(layered))

        # Assert watermarked PDF file exists
        self.assertTrue(os.path.exists(added))

        # Assert watermarked PDF has page resources
        self.assertTrue(Info(added).resources())
        return added

    @Timer.decorator
    def test_conduit_watermark_label(self):
        """Apply a watermark label to a PDF file."""
        label = os.path.basename(self.pdf_path)
        labeled = Label(self.pdf_path, label, tempdir=self.temp.name, suffix=None).write(cleanup=False)

        # Assert watermarked PDF file exists
        self.assertTrue(os.path.exists(labeled))

        # Assert watermarked PDF has page resources
        self.assertTrue(Info(labeled).resources())
        return labeled


if __name__ == '__main__':
    unittest.main()



# Upscale a PDF file
import os
from tempfile import NamedTemporaryFile

from PyPDF3 import PdfFileReader, PdfFileWriter
from PyPDF3.pdf import PageObject
from pdfrw import PdfReader, PdfWriter, PageMerge, IndirectPdfDict

from pdfconduit.utils.info import Info
from pdfconduit.utils.path import add_suffix


class Upscale:
    def __init__(self, file_name, margin_x=0, margin_y=0, scale=1.5, suffix='scaled', tempdir=None, method='pdfrw'):
        self.file_name = file_name
        self.margin_x = margin_x
        self.margin_y = margin_y
        self.scale = scale
        self.suffix = suffix

        # Set output file name
        if tempdir:
            with NamedTemporaryFile(suffix='_' + suffix + '.pdf', dir=tempdir, delete=False) as temp:
                self.output = temp.name
        elif suffix:
            self.output = os.path.join(os.path.dirname(file_name), add_suffix(file_name, suffix))
        else:
            self.output = NamedTemporaryFile(suffix='_' + suffix + '.pdf').name

        # Get target width and height
        dims = Info(self.file_name).dimensions
        self.target_w = dims['w'] * self.scale
        self.target_h = dims['h'] * self.scale

        # Execute either pdfrw or PyPDF3 method
        if method == 'pypdf3':
            self.pypdf3()
        else:
            self.pdfrw()

    def __str__(self):
        return self.file

    @property
    def file(self):
        return str(self.output)

    def _pdfrw_adjust(self, page):
        info = PageMerge().add(page)
        x1, y1, x2, y2 = info.xobj_box
        viewrect = (self.margin_x, self.margin_y, x2 - x1 - 2 * self.margin_x, y2 - y1 - 2 * self.margin_y)
        page = PageMerge().add(page, viewrect=viewrect)
        page[0].scale(self.scale)
        return page.render()

    def pdfrw(self):
        reader = PdfReader(self.file_name)
        writer = PdfWriter(self.output)
        for i in list(range(0, len(reader.pages))):
            writer.addpage(self._pdfrw_adjust(reader.pages[i]))
        writer.trailer.Info = IndirectPdfDict(reader.Info or {})
        writer.write()

    def pypdf3(self):
        reader = PdfFileReader(self.file_name)
        writer = PdfFileWriter()

        # Number of pages in input document
        page_count = reader.getNumPages()

        for page_number in range(page_count):
            wtrmrk = reader.getPage(page_number)

            page = PageObject.createBlankPage(width=self.target_w, height=self.target_h)
            page.mergeScaledTranslatedPage(wtrmrk, self.scale, self.margin_x, self.margin_y)
            writer.addPage(page)

        with open(self.output, "wb") as outputStream:
            writer.write(outputStream)
        return self.output


def upscale(file_name, margin_x=0, margin_y=0, scale=1.5, suffix='scaled', tempdir=None, method='pdfrw'):
    return str(Upscale(file_name, margin_x, margin_y, scale, suffix, tempdir, method))



import os
import unittest
from tempfile import TemporaryDirectory

from looptools import Timer

from pdfconduit import Info, Merge, Upscale, Rotate, slicer
from tests import *


class TestTransformMerge(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.pdfs = [os.path.join(test_data_dir, p) for p in ['article.pdf', 'charts.pdf', 'document.pdf', 'manual.pdf']]

    def setUp(self):
        self.temp = TemporaryDirectory()

    def tearDown(self):
        self.temp.cleanup()

    @Timer.decorator
    def test_merge_pypdf3(self):
        """Merge multiple PDF files into a single PDF using the `PyPDF3` library."""
        merged = Merge(self.pdfs, output_name='merged_pypdf3', output_dir=self.temp.name, method='pypdf3')

        # Assert merged file exists
        self.assertTrue(os.path.exists(merged.file))

        # Assert sum of pages in original pdf files equals sum of pages in merged pdf
        self.assertEqual(sum([Info(pdf).pages for pdf in self.pdfs]), Info(merged.file).pages)
        return merged

    @Timer.decorator
    def test_merge_pdfrw(self):
        """Merge multiple PDF files into a single PDF using the `pdfrw` library."""
        merged = Merge(self.pdfs, output_name='merged_pdfrw', output_dir=self.temp.name, method='pdfrw')

        # Assert merged file exists
        self.assertTrue(os.path.exists(merged.file))

        # Assert sum of pages in original pdf files equals sum of pages in merged pdf
        self.assertEqual(sum([Info(pdf).pages for pdf in self.pdfs]), Info(merged.file).pages)
        return merged


class TestTransformUpscale(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.pdf_path = pdf_path
        cls.temp = TemporaryDirectory()

    @classmethod
    def tearDownClass(cls):
        cls.temp.cleanup()

    @Timer.decorator
    def test_upscale_pdfrw_20x(self):
        """Resize a PDF file to 2.0x times the original scale."""
        s = 2.0
        upscaled = Upscale(pdf_path, scale=s, suffix='upscaled_2.0_pdfrw', tempdir=self.temp.name, method='pdfrw').file

        # Assert upscaled file exists
        self.assertTrue(os.path.isfile(upscaled))

        # Assert upscaled pdf file is the correct size
        self.assertEqual(Info(upscaled).size, tuple([i * s for i in Info(pdf_path).size]))
        return upscaled

    @Timer.decorator
    def test_upscale_pdfrw_15x(self):
        """Resize a PDF file to 1.5x times the original scale."""
        s = 1.5
        upscaled = Upscale(pdf_path, scale=s, suffix='upscaled_1.5_pdfrw', tempdir=self.temp.name, method='pdfrw').file

        # Assert upscaled file exists
        self.assertTrue(os.path.isfile(upscaled))

        # Assert upscaled pdf file is the correct size
        self.assertEqual(Info(upscaled).size, tuple([i * s for i in Info(pdf_path).size]))
        return upscaled

    @Timer.decorator
    def test_upscale_pdfrw_30x(self):
        """Resize a PDF file to 3.0x times the original scale."""
        s = 3.0
        upscaled = Upscale(pdf_path, scale=s, suffix='upscaled_3.0_pdfrw', tempdir=self.temp.name, method='pdfrw').file

        # Assert upscaled file exists
        self.assertTrue(os.path.isfile(upscaled))

        # Assert upscaled pdf file is the correct size
        self.assertEqual(Info(upscaled).size, tuple([i * s for i in Info(pdf_path).size]))
        return upscaled

    @Timer.decorator
    def test_downscale_pdfrw_20x(self):
        """Resize a PDF file to 3.0x times the original scale."""
        s = 1 / 2
        upscaled = Upscale(pdf_path, scale=s, suffix='downscaled_2.0_pdfrw', tempdir=self.temp.name,
                           method='pdfrw').file

        # Assert upscaled file exists
        self.assertTrue(os.path.isfile(upscaled))

        # Assert upscaled pdf file is the correct size
        self.assertEqual(Info(upscaled).size, tuple([i * s for i in Info(pdf_path).size]))
        return upscaled


class TestTransformSlice(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.pdf_path = pdf_path

    def setUp(self):
        self.temp = TemporaryDirectory()

    def tearDown(self):
        self.temp.cleanup()

    @Timer.decorator
    def test_slice(self):
        """Slice a page range from a PDF to create a new 'trimmed' pdf file."""
        fp = 1
        lp = 1
        sliced = slicer(self.pdf_path, first_page=fp, last_page=lp, tempdir=self.temp.name)

        # Assert sliced file exists
        self.assertTrue(os.path.isfile(sliced))

        # Confirm slicer sliced the correct number of pages
        self.assertEqual(Info(sliced).pages, len(range(fp, lp + 1)))
        return sliced

    @Timer.decorator
    def test_slice2(self):
        """Slice a page range from a PDF to create a new 'trimmed' pdf file."""
        fp = 4
        lp = 7
        sliced = slicer(self.pdf_path, first_page=fp, last_page=lp, tempdir=self.temp.name)

        # Assert sliced file exists
        self.assertTrue(os.path.isfile(sliced))

        # Confirm slicer sliced the correct number of pages
        self.assertEqual(Info(sliced).pages, len(range(fp, lp + 1)))
        return sliced


class TestRotate(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.pdf_path = pdf_path

    def setUp(self):
        self.temp = TemporaryDirectory()

    def tearDown(self):
        self.temp.cleanup()

    @Timer.decorator
    def test_rotate_pdfrw_90(self):
        """Rotate a PDF file by 90 degrees using the `pdfrw` library."""
        rotation = 90
        rotated = Rotate(self.pdf_path, rotation, suffix='rotated_pdfrw', tempdir=self.temp.name, method='pdfrw').file

        # Assert rotated pdf file exists
        self.assertTrue(os.path.isfile(rotated))

        # Assert pdf file was rotated by the correct amount of degrees
        self.assertEqual(Info(rotated).rotate, rotation)
        return rotated

    @Timer.decorator
    def test_rotate_pdfrw_180(self):
        """Rotate a PDF file by 180 degrees using the `pdfrw` library."""
        rotation = 180
        rotated = Rotate(self.pdf_path, rotation, suffix='rotated_180_pdfrw', tempdir=self.temp.name,
                         method='pdfrw').file

        # Assert rotated pdf file exists
        self.assertTrue(os.path.isfile(rotated))

        # Assert pdf file was rotated by the correct amount of degrees
        self.assertEqual(Info(rotated).rotate, rotation)
        return rotated

    @Timer.decorator
    def test_rotate_pdfrw_270(self):
        """Rotate a PDF file by 270 degrees using the `pdfrw` library."""
        rotation = 270
        rotated = Rotate(self.pdf_path, rotation, suffix='rotated_270_pdfrw', tempdir=self.temp.name,
                         method='pdfrw').file

        # Assert rotated pdf file exists
        self.assertTrue(os.path.isfile(rotated))

        # Assert pdf file was rotated by the correct amount of degrees
        self.assertEqual(Info(rotated).rotate, rotation)
        return rotated

    @Timer.decorator
    def test_rotate_pypdf3_90(self):
        """Rotate a PDF file by 90 degrees using the `pypdf3` library."""
        rotation = 90
        rotated = Rotate(self.pdf_path, rotation, suffix='rotated_pdfrw', tempdir=self.temp.name, method='pypdf3').file

        # Assert rotated pdf file exists
        self.assertTrue(os.path.isfile(rotated))

        # Assert pdf file was rotated by the correct amount of degrees
        self.assertEqual(Info(rotated).rotate, rotation)
        return rotated

    @Timer.decorator
    def test_rotate_pypdf3_180(self):
        """Rotate a PDF file by 180 degrees using the `pypdf3` library."""
        rotation = 180
        rotated = Rotate(self.pdf_path, rotation, suffix='rotated_180_pdfrw', tempdir=self.temp.name,
                         method='pypdf3').file

        # Assert rotated pdf file exists
        self.assertTrue(os.path.isfile(rotated))

        # Assert pdf file was rotated by the correct amount of degrees
        self.assertEqual(Info(rotated).rotate, rotation)
        return rotated

    @Timer.decorator
    def test_rotate_pypdf3_270(self):
        """Rotate a PDF file by 270 degrees using the `pypdf3` library."""
        rotation = 270
        rotated = Rotate(self.pdf_path, rotation, suffix='rotated_270_pdfrw', tempdir=self.temp.name,
                         method='pypdf3').file

        # Assert rotated pdf file exists
        self.assertTrue(os.path.isfile(rotated))

        # Assert pdf file was rotated by the correct amount of degrees
        self.assertEqual(Info(rotated).rotate, rotation)
        return rotated


if __name__ == '__main__':
    unittest.main()



import os
import unittest
from tempfile import TemporaryDirectory

from looptools import Timer

from pdfconduit.convert import IMG2PDF, PDF2IMG
from pdfconduit import Info, Flatten
from tests import *


class TestConvertFlatten(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.pdf_path = pdf_path
        cls.flat = None

    def tearDown(self):
        if os.path.exists(self.flat):
            os.remove(self.flat)

    @Timer.decorator
    def test_flatten(self):
        """Create a 'flattened' pdf file without layers."""
        flat = Flatten(self.pdf_path, suffix='flat').save()

        # Assert pdf file exists
        self.assertTrue(os.path.exists(flat))

        # Assert there are the same number of pages in the 'original' and 'flattened' pdf
        self.assertEqual(Info(self.pdf_path).pages, Info(flat).pages)

        # Confirm that PDF page sizes have not increased
        self.assertTrue(abs(Info(self.pdf_path).size[0] / Info(flat).size[0]) <= 1)
        self.assertTrue(abs(Info(self.pdf_path).size[1] / Info(flat).size[1]) <= 1)
        self.flat = flat
        return flat


class TestConvertImg2Pdf(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.img_path = img_path
        cls.pdf = None
        cls.tempdir = TemporaryDirectory()

    @classmethod
    def tearDownClass(cls):
        if hasattr(cls, 'tempdir'):
            cls.tempdir.cleanup()

    def tearDown(self):
        if os.path.exists(self.pdf):
            os.remove(self.pdf)

    @Timer.decorator
    def test_convert(self):
        """Convert an image file into PDF."""
        ip = IMG2PDF(tempdir=self.tempdir)
        self.pdf = ip.convert(self.img_path)

        # Assert pdf file exists
        self.assertTrue(os.path.exists(self.pdf))
        return self.pdf

    @Timer.decorator
    def test_convert_packet(self):
        """Convert an image file into PDF."""
        self.pdf = IMG2PDF([self.img_path, self.img_path, self.img_path],
                           destination=test_data_dir,
                           tempdir=self.tempdir).save(clean_temp=False)

        # Assert pdf file exists
        self.assertTrue(os.path.exists(self.pdf))
        return self.pdf


class TestConvertPdf2Img(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.pdf_path = os.path.join(test_data_dir, 'plan_p.pdf')
        cls.img = None

    def tearDown(self):
        if os.path.exists(self.img):
            os.remove(self.img)

    @Timer.decorator
    def test_pdf2img(self):
        """Convert a PDF file to a png image."""
        img = PDF2IMG(self.pdf_path).save()

        # Assert img file exists
        self.assertTrue(os.path.exists(img[0]))

        # Assert img file is correct file type
        self.assertTrue(img[0].endswith('.png'))
        self.img = img[0]
        return img[0]


if __name__ == '__main__':
    unittest.main()



# Encrypt a PDF file with password protection
from PyPDF3 import PdfFileWriter

from pdfconduit.utils import add_suffix, pypdf3_reader


class Encrypt:
    def __init__(self,
                 pdf,
                 user_pw,
                 owner_pw=None,
                 output=None,
                 suffix='secured',
                 bit128=True,
                 allow_printing=True,
                 allow_commenting=False,
                 overwrite_permission=None,
                 progress_bar_enabled=False,
                 progress_bar='gui',
                 decrypt=None):
        """Password protect PDF file and allow all other permissions."""
        self.pdf = pdf
        self.user_pw = user_pw
        self.owner_pw = owner_pw
        self.output = add_suffix(pdf, suffix=suffix) if not output else output
        self.encrypt_128 = bit128
        self.allow_printing = allow_printing
        self.allow_commenting = allow_commenting
        self.overwrite_permission = overwrite_permission
        self.progress_bar_enabled = progress_bar_enabled
        self.progress_bar = progress_bar

        self.encrypt(decrypt)

    def __str__(self):
        return str(self.output)

    def encrypt(self, decrypt=None):
        # Create PDF writer object
        pdf_writer = PdfFileWriter()
        with open(self.pdf, 'rb') as pdf_file:
            # Read opened PDF file
            pdf_reader = pypdf3_reader(pdf_file, decrypt)

            # Add each page from source PDF
            for page_num in range(pdf_reader.numPages):
                page = pdf_reader.getPage(page_num)
                pdf_writer.addPage(page)

            # Apply encryption to writer object
            pdf_writer.encrypt(self.user_pw,
                               self.owner_pw,
                               use_128bit=self.encrypt_128,
                               allow_printing=self.allow_printing,
                               allow_commenting=self.allow_commenting,
                               overwrite_permission=self.overwrite_permission)

            # todo: add metadata adding functionality
            pdf_writer.addMetadata({
                '/Producer': 'pdfconduit',
                '/Creator': 'HPA Design',
                '/Author': 'HPA Design',
            })

            # Write encrypted PDF to file
            with open(self.output, 'wb') as output_pdf:
                pdf_writer.write(output_pdf,
                                 progress_bar=self.progress_bar,
                                 progress_bar_enabled=self.progress_bar_enabled)
        return self.output


def main():
    try:
        from pdfconduit import GUI
        GUI.encrypt()
    except ImportError:
        from argparse import ArgumentParser
        # TODO: Finish implementing argparse

        # Declare argparse argument descriptions
        usage = 'Encrypt PDF files'
        description = 'Password protect a PDF file by applying encryption.'
        helpers = {
            'pdf': "PDF file to encrypt",
            'p': "Retrieve Python interpreter information such as Python version and Python compiler",
            's': "Retrieve software related information such as operating system, machine type, username, etc.",
            'hw': "Retrieve Hardware related information such as memory and processor information",
        }

        # construct the argument parse and parse the arguments
        ap = ArgumentParser(usage=usage, description=description)
        ap.add_argument("pdf", help=helpers['pdf'])
        ap.add_argument("-upw", '--user_pw', help=helpers['upw'])
        ap.add_argument("-opw", '--owner_pw', help=helpers['opw'])
        ap.add_argument("-out", '--output', help=helpers['out'])
        args = vars(ap.parse_args())


if __name__ == '__main__':
    main()
